@@ -1,7 +1,7 @@
-#' Read a genotype matrix in plink BED format
+#' Read a genotype matrix in Plink BED format
 #'
-#' This function reads genotypes encoded in a plink-formatted BED (binary) file, returning them in a standard R matrix containing genotypes (values in \code{c(0,1,2,NA)}).
-#' Each genotype per locus (m loci) and individual (n total) counts the number of alternative alleles or \code{NA} for missing data.
+#' This function reads genotypes encoded in a Plink-formatted BED (binary) file, returning them in a standard R matrix containing genotypes encoded numerically as dosages (values in `c( 0, 1, 2, NA )`).
+#' Each genotype per locus (`m` loci) and individual (`n` total) counts the number of reference alleles, or `NA` for missing data.
 #' No *.fam or *.bim files are read by this basic function.
 #' Since BED does not encode the data dimensions internally, these values must be provided by the user.
 #'
@@ -11,25 +11,25 @@
 #'
 #' This function only supports locus-major BED files, which are the standard for modern data.
 #' Format is validated via the BED file's magic numbers (first three bytes of file).
-#' Older BED files can be converted using plink.
+#' Older BED files can be converted using Plink.
 #' 
 #' @param file Input file path.
 #' *.bed extension may be omitted (will be added automatically if it is missing).
 #' @param names_loci Vector of loci names, to become the row names of the genotype matrix.
-#' If provided, its length sets \code{m_loci} below.
-#' If \code{NULL}, the returned genotype matrix will not have row names, and \code{m_loci} must be provided.
+#' If provided, its length sets `m_loci` below.
+#' If `NULL`, the returned genotype matrix will not have row names, and `m_loci` must be provided.
 #' @param names_ind Vector of individual names, to become the column names of the genotype matrix.
-#' If provided, its length sets \code{n_ind} below.
-#' If \code{NULL}, the returned genotype matrix will not have column names, and \code{n_ind} must be provided.
+#' If provided, its length sets `n_ind` below.
+#' If `NULL`, the returned genotype matrix will not have column names, and `n_ind` must be provided.
 #' @param m_loci Number of loci in the input genotype table.
-#' Required if \code{names_loci = NULL}, as its value is not inferrable from the BED file itself.
-#' Ignored if \code{names_loci} is provided.
+#' Required if `names_loci = NULL`, as its value is not deducible from the BED file itself.
+#' Ignored if `names_loci` is provided.
 #' @param n_ind Number of individuals in the input genotype table.
-#' Required if \code{names_ind = NULL}, as its value is not inferrable from the BED file itself.
-#' Ignored if \code{names_ind} is provided.
-#' @param verbose If TRUE (default) function reports the path of the file being read (after autocompleting the extension).
+#' Required if `names_ind = NULL`, as its value is not deducible from the BED file itself.
+#' Ignored if `names_ind` is provided.
+#' @param verbose If `TRUE` (default) function reports the path of the file being read (after autocompleting the extension).
 #'
-#' @return The \eqn{m \times n}{m-by-n} genotype matrix.
+#' @return The `m`-by-`n` genotype matrix.
 #'
 #' @examples
 #' # first obtain data dimensions from BIM and FAM files
@@ -41,7 +41,7 @@
 #' bim <- read_bim(file_bim)
 #' fam <- read_fam(file_fam)
 #' 
-#' # read an existing plink *.bim file
+#' # read an existing Plink *.bim file
 #' # pass locus and individual IDs as vectors, setting data dimensions too
 #' X <- read_bed(file_bed, bim$id, fam$id)
 #' X
@@ -53,10 +53,12 @@
 #' X
 #' 
 #' @seealso
-#' \code{\link{read_plink}} for reading a set of BED/BIM/FAM files.
+#' [read_plink()] for reading a set of BED/BIM/FAM files.
+#'
+#' [geno_to_char()] for translating numerical genotypes into more human-readable character encodings.
 #' 
 #' Plink BED format reference:
-#' \url{https://www.cog-genomics.org/plink/1.9/formats#bed}
+#' <https://www.cog-genomics.org/plink/1.9/formats#bed>
 #'
 #' @export
 read_bed <- function(file, names_loci = NULL, names_ind = NULL, m_loci = NA, n_ind = NA, verbose = TRUE) {
@@ -87,6 +89,11 @@ read_bed <- function(file, names_loci = NULL, names_ind = NULL, m_loci = NA, n_i
 
     # C++ doesn't work with tildes in names, so let's expand path before we pass to C++
     file <- path.expand(file)
+
+    # before passing it to c++ function, make sure it exists
+    # C++ does its own check, but on very long paths the buffer overflows
+    if ( !file.exists( file ) )
+        stop( 'File does not exist: ', file )
     
     # read in Rcpp!
     X <- read_bed_cpp(file, m_loci, n_ind)
